@@ -1,7 +1,3 @@
-#if defined (HAVE_CONFIG_H)
-#  include "config.h"
-#endif
-
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -10,28 +6,39 @@
 const char *cdiStringError(int cdiErrno)
 {
   static const char UnknownError[] = "Unknown Error";
+  static const char _ETMOF[]       = "Too many open files";
+  static const char _EISDIR[]      = "Is a directory";
+  static const char _EISEMPTY[]    = "File is empty";
   static const char _EUFTYPE[]     = "Unsupported file type";
   static const char _ELIBNAVAIL[]  = "Unsupported file type (library support not compiled in)";
   static const char _EUFSTRUCT[]   = "Unsupported file structure";
-  static const char _EUNC4[]       = "Unsupported netCDF4 structure";
+  static const char _EUNC4[]       = "Unsupported NetCDF4 structure";
+  static const char _EDIMSIZE[]    = "Invalid dimension size";
   static const char _ELIMIT[]      = "Internal limits exceeded";
+  static const char _EINVAL[]      = "Invalid argument";
 
   switch (cdiErrno) {
   case CDI_ESYSTEM:
     {
-      char *cp = (char *) strerror(errno);
+      const char *cp = strerror(errno);
       if ( cp == NULL ) break;
       return cp;
     }
+  case CDI_ETMOF:      return _ETMOF;
+  case CDI_EISDIR:     return _EISDIR;
+  case CDI_EISEMPTY:   return _EISEMPTY;
   case CDI_EUFTYPE:    return _EUFTYPE;
   case CDI_ELIBNAVAIL: return _ELIBNAVAIL;
   case CDI_EUFSTRUCT:  return _EUFSTRUCT;
   case CDI_EUNC4:      return _EUNC4;
+  case CDI_EDIMSIZE:   return _EDIMSIZE;
   case CDI_ELIMIT:     return _ELIMIT;
+  case CDI_EINVAL:     return _EINVAL;
   }
 
   return UnknownError;
 }
+
 /*
  * Local Variables:
  * c-file-style: "Java"
