@@ -1,7 +1,3 @@
-#if defined (HAVE_CONFIG_H)
-#  include "config.h"
-#endif
-
 #include <assert.h>
 #include <limits.h>
 
@@ -9,17 +5,12 @@
 #include "cdi.h"
 #include "cdi_int.h"
 #include "resource_handle.h"
-#include "resource_handle.h"
 #include "resource_unpack.h"
 #include "namespace.h"
 #include "serialize.h"
 #include "institution.h"
 
 
-static int ECMWF  = CDI_UNDEFID,
-  MPIMET = CDI_UNDEFID,
-  MCH    = CDI_UNDEFID;
-
 typedef struct
 {
   int    self;
@@ -49,7 +40,7 @@ static const resOps instituteOps = {
 };
 
 static
-void instituteDefaultValue ( institute_t * instituteptr )
+void instituteDefaultValue(institute_t *instituteptr)
 {
   instituteptr->self       = CDI_UNDEFID;
   instituteptr->used       = 0;
@@ -59,37 +50,36 @@ void instituteDefaultValue ( institute_t * instituteptr )
   instituteptr->longname   = NULL;
 }
 
-void instituteDefaultEntries ( void )
+void instituteDefaultEntries(void)
 {
   cdiResH resH[]
-    = { ECMWF   = institutDef( 98,   0, "ECMWF",     "European Centre for Medium-Range Weather Forecasts"),
-        MPIMET  = institutDef( 98, 232, "MPIMET",    "Max-Planck-Institute for Meteorology"),
-        institutDef( 98, 255, "MPIMET",    "Max-Planck-Institute for Meteorology"),
+    = { institutDef( 98,   0, "ECMWF",     "European Centre for Medium-Range Weather Forecasts"),
+        institutDef(252,   1, "MPIMET",    "Max Planck Institute for Meteorology"),
         institutDef( 98, 232, "MPIMET",    "Max Planck Institute for Meteorology"),
-        institutDef( 78,   0, "DWD",       "Deutscher Wetterdienst"),
+        institutDef( 98, 255, "MPIMET",    "Max-Planck-Institute for Meteorology"),
         institutDef( 78, 255, "DWD",       "Deutscher Wetterdienst"),
-        MCH     = institutDef(215, 255, "MCH",       "MeteoSwiss"),
+        institutDef( 78,   0, "DWD",       "Deutscher Wetterdienst"),
+        institutDef(215, 255, "MCH",       "MeteoSwiss"),
         institutDef(  7,   0, "NCEP",      "National Centers for Environmental Prediction"),
         institutDef(  7,   1, "NCEP",      "National Centers for Environmental Prediction"),
         institutDef( 60,   0, "NCAR",      "National Center for Atmospheric Research"),
         institutDef( 74,   0, "METOFFICE", "U.K. Met Office"),
         institutDef( 97,   0, "ESA",       "European Space Agency"),
         institutDef( 99,   0, "KNMI",      "Royal Netherlands Meteorological Institute"),
+        institutDef( 80,   0, "CNMC",      "Reparto per la Meteorologia, Rome (REMET)"),
+        // institutDef(  0,   0, "IPSL", "IPSL (Institut Pierre Simon Laplace, Paris, France)");
   };
-  /*     (void) institutDef(  0,   0, "IPSL", "IPSL (Institut Pierre Simon Laplace, Paris, France)"); */
-
-  size_t n = sizeof(resH)/sizeof(*resH);
 
+  const size_t n = sizeof(resH)/sizeof(*resH);
   for (size_t i = 0; i < n ; i++ )
     reshSetStatus(resH[i], &instituteOps, RESH_IN_USE);
 }
 
 
 static int
-instituteCompareKernel(institute_t *  ip1, institute_t * ip2)
+instituteCompareKernel(institute_t *ip1, institute_t *ip2)
 {
   int differ = 0;
-  size_t len1, len2;
 
   if ( ip1->name )
     {
@@ -100,8 +90,8 @@ instituteCompareKernel(institute_t *  ip1, institute_t * ip2)
         {
           if ( ip2->name )
             {
-              len1 = strlen(ip1->name);
-              len2 = strlen(ip2->name);
+              const size_t len1 = strlen(ip1->name);
+              const size_t len2 = strlen(ip2->name);
               if ( (len1 != len2) || memcmp(ip2->name, ip1->name, len2) ) differ = 1;
             }
         }
@@ -110,15 +100,16 @@ instituteCompareKernel(institute_t *  ip1, institute_t * ip2)
     {
       if ( ip2->longname )
         {
-          len1 = strlen(ip1->longname);
-          len2 = strlen(ip2->longname);
-          if ( (len1 < len2) || memcmp(ip2->longname, ip1->longname, len2) ) differ = 1;
+          const size_t len1 = strlen(ip1->longname);
+          const size_t len2 = strlen(ip2->longname);
+          if ( (len1 != len2) || memcmp(ip2->longname, ip1->longname, len2) ) differ = 1;
         }
     }
   else
     {
       if ( !( ip2->center    == ip1->center &&
               ip2->subcenter == ip1->subcenter )) differ = 1;
+      if (ip1->subcenter > 0 && ip1->subcenter != 255 && ip2->subcenter != ip1->subcenter) differ = 1;
     }
 
   return differ;
@@ -134,8 +125,8 @@ struct instLoc
 static enum cdiApplyRet
 findInstitute(int id, void *res, void *data)
 {
-  institute_t * ip1 = ((struct instLoc *)data)->ip;
-  institute_t * ip2 = (institute_t*) res;
+  institute_t *ip1 = ((struct instLoc *)data)->ip;
+  institute_t *ip2 = (institute_t*) res;
   if (ip2->used && !instituteCompareKernel(ip1, ip2))
     {
       ((struct instLoc *)data)->id = id;
@@ -148,19 +139,17 @@ findInstitute(int id, void *res, void *data)
 
 int institutInq(int center, int subcenter, const char *name, const char *longname)
 {
-  institute_t * ip_ref = (institute_t *) Malloc(sizeof (*ip_ref));
-  ip_ref->self       = CDI_UNDEFID;
-  ip_ref->used       = 0;
-  ip_ref->center     = center;
-  ip_ref->subcenter  = subcenter;
-  ip_ref->name       = name && name[0] ? (char *)name : NULL;
-  ip_ref->longname   = longname && longname[0] ? (char *)longname : NULL;
+  institute_t ip_ref;
+  ip_ref.self       = CDI_UNDEFID;
+  ip_ref.used       = 0;
+  ip_ref.center     = center;
+  ip_ref.subcenter  = subcenter;
+  ip_ref.name       = (name && name[0]) ? (char *)name : NULL;
+  ip_ref.longname   = (longname && longname[0]) ? (char *)longname : NULL;
 
-  struct instLoc state = { .ip = ip_ref, .id = CDI_UNDEFID };
+  struct instLoc state = { .ip = &ip_ref, .id = CDI_UNDEFID };
   cdiResHFilterApply(&instituteOps, findInstitute, &state);
 
-  Free(ip_ref);
-
   return state.id;
 }
 
@@ -190,8 +179,7 @@ institute_t *instituteNewEntry(cdiResH resH, int center, int subcenter,
 
 int institutDef(int center, int subcenter, const char *name, const char *longname)
 {
-  institute_t * instituteptr
-    = instituteNewEntry(CDI_UNDEFID, center, subcenter, name, longname);
+  institute_t *instituteptr = instituteNewEntry(CDI_UNDEFID, center, subcenter, name, longname);
   return instituteptr->self;
 }
 
@@ -328,10 +316,9 @@ int instituteUnpack(void *buf, int size, int *position, int originNamespace,
 {
   int tempbuf[institute_nints];
   int instituteID;
-  char *name, *longname;
   serializeUnpack(buf, size, position, tempbuf, institute_nints, CDI_DATATYPE_INT, context);
-  name = (char *) Malloc((size_t)tempbuf[3] + (size_t)tempbuf[4]);
-  longname = name + tempbuf[3];
+  char *name = (char *) Malloc((size_t)tempbuf[3] + (size_t)tempbuf[4]);
+  char *longname = name + tempbuf[3];
   serializeUnpack(buf, size, position, name, tempbuf[3], CDI_DATATYPE_TXT, context);
   serializeUnpack(buf, size, position, longname, tempbuf[4], CDI_DATATYPE_TXT, context);
   int targetID = namespaceAdaptKey(tempbuf[0], originNamespace);
