@@ -2,22 +2,22 @@
 #define _STREAM_IEG_H
 
 #ifndef _IEG_H
-#  include "ieg.h"
+#include "ieg.h"
 #endif
 
 int    iegInqContents(stream_t *streamptr);
 int    iegInqTimestep(stream_t *streamptr, int tsID);
 
 int    iegInqRecord(stream_t *streamptr, int *varID, int *levelID);
-int    iegDefRecord(stream_t *streamptr);
-int    iegCopyRecord(stream_t *streamptr2, stream_t *streamptr1);
-int    iegReadRecord(stream_t *streamptr, double *data, int *nmiss);
-int    iegWriteRecord(stream_t *streamptr, const double *data);
+void   iegDefRecord(stream_t *streamptr);
+void   iegCopyRecord(stream_t *streamptr2, stream_t *streamptr1);
+void   ieg_read_record(stream_t *streamptr, int memtype, void *data, size_t *nmiss);
+void   ieg_write_record(stream_t *streamptr, int memtype, const void *data);
 
-void   iegReadVarDP (stream_t *streamptr, int varID,       double *data, int *nmiss);
+void   iegReadVarDP (stream_t *streamptr, int varID,       double *data, size_t *nmiss);
 void   iegWriteVarDP(stream_t *streamptr, int varID, const double *data);
 
-void   iegReadVarSliceDP (stream_t *streamptr, int varID, int levelID,       double *data, int *nmiss);
+void   iegReadVarSliceDP (stream_t *streamptr, int varID, int levelID,       double *data, size_t *nmiss);
 void   iegWriteVarSliceDP(stream_t *streamptr, int varID, int levelID, const double *data);
 
 #endif  /* _STREAM_IEG_H */
