@@ -1,12 +1,7 @@
-#if defined (HAVE_CONFIG_H)
-#  include "config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
 #endif
 
-#include <limits.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "dmemory.h"
 
 #include "error.h"
@@ -16,18 +11,13 @@
 #include "varscan.h"
 #include "datetime.h"
 #include "service.h"
+#include "stream_scan.h"
 #include "stream_srv.h"
-#include "vlist.h"
+#include "get_num_missvals.h"
 #include "exse.h"
 
 
-#if defined (HAVE_LIBSERVICE)
-
-typedef struct {
-  int param;
-  int level;
-} srvcompvar_t;
-
+#ifdef HAVE_LIBSERVICE
 
 static
 int srvInqDatatype(int prec)
@@ -84,11 +74,13 @@ int srvInqRecord(stream_t *streamptr, int *varID, int *levelID)
 }
 */
 
-void srvReadRecord(stream_t *streamptr, double *data, int *nmiss)
+static
+void srv_read_recordSP(stream_t *streamptr, float *data, size_t *nmiss)
 {
   int vlistID  = streamptr->vlistID;
   int fileID   = streamptr->fileID;
   int tsID     = streamptr->curTsID;
+
   int vrecID   = streamptr->tsteps[tsID].curRecID;
   int recID    = streamptr->tsteps[tsID].recIDs[vrecID];
   int varID    = streamptr->tsteps[tsID].records[recID].varID;
@@ -97,27 +89,56 @@ void srvReadRecord(stream_t *streamptr, double *data, int *nmiss)
   fileSetPos(fileID, recpos, SEEK_SET);
 
   void *srvp = streamptr->record->exsep;
-  int status = srvRead(fileID, srvp);
-  if ( status != 0 )
-    Error("Failed to read record from SRV file");
+  if (srvRead(fileID, srvp) < 0) Error("Failed to read record from SRV file");
+
+  int header[8];
+  srvInqHeader(srvp, header);
+  srvInqDataSP(srvp, data);
+
+  double missval = vlistInqVarMissval(vlistID, varID);
+  size_t size = gridInqSize(vlistInqVarGrid(vlistID, varID));
+
+  *nmiss = get_num_missvalsSP(size, data, (float)missval);
+
+  streamptr->numvals += size;
+}
+
+static
+void srv_read_recordDP(stream_t *streamptr, double *data, size_t *nmiss)
+{
+  int vlistID  = streamptr->vlistID;
+  int fileID   = streamptr->fileID;
+  int tsID     = streamptr->curTsID;
+
+  int vrecID   = streamptr->tsteps[tsID].curRecID;
+  int recID    = streamptr->tsteps[tsID].recIDs[vrecID];
+  int varID    = streamptr->tsteps[tsID].records[recID].varID;
+  off_t recpos = streamptr->tsteps[tsID].records[recID].position;
+
+  fileSetPos(fileID, recpos, SEEK_SET);
+
+  void *srvp = streamptr->record->exsep;
+  if (srvRead(fileID, srvp) < 0) Error("Failed to read record from SRV file");
 
   int header[8];
   srvInqHeader(srvp, header);
   srvInqDataDP(srvp, data);
 
   double missval = vlistInqVarMissval(vlistID, varID);
-  int gridID  = vlistInqVarGrid(vlistID, varID);
-  int size    = gridInqSize(gridID);
+  size_t size = gridInqSize(vlistInqVarGrid(vlistID, varID));
+
+  *nmiss = get_num_missvalsDP(size, data, missval);
 
   streamptr->numvals += size;
+}
 
-  *nmiss = 0;
-  for ( int i = 0; i < size; i++ )
-    if ( DBL_IS_EQUAL(data[i], missval) || DBL_IS_EQUAL(data[i], (float)missval) )
-      {
-	data[i] = missval;
-	(*nmiss)++;
-      }
+
+void srv_read_record(stream_t *streamptr, int memtype, void *data, size_t *nmiss)
+{
+  if (memtype == MEMTYPE_DOUBLE)
+    srv_read_recordDP(streamptr, (double*) data, nmiss);
+  else
+    srv_read_recordSP(streamptr, (float*) data, nmiss);
 }
 
 
@@ -130,7 +151,6 @@ void srvCopyRecord(stream_t *streamptr2, stream_t *streamptr1)
 void srvDefRecord(stream_t *streamptr)
 {
   Record *record = streamptr->record;
-  srvrec_t *srvp = (srvrec_t*) record->exsep;
 
   int pdis, pcat, pnum;
   cdiDecodeParam(record->param, &pnum, &pcat, &pdis);
@@ -142,45 +162,62 @@ void srvDefRecord(stream_t *streamptr)
   header[3] = record->time;
 
   int gridID = record->gridID;
-  int xsize = gridInqXsize(gridID),
-      ysize = gridInqYsize(gridID);
+  size_t xsize = gridInqXsize(gridID),
+         ysize = gridInqYsize(gridID);
   if ( xsize == 0 || ysize == 0 )
     {
       xsize = gridInqSize(gridID);
       ysize = 1;
     }
   if ( gridInqType(gridID) == GRID_UNSTRUCTURED ) ysize = 1;
-  if ( gridInqSize(gridID) != xsize*ysize )
+  if ( (size_t)gridInqSize(gridID) != xsize*ysize )
     Error("Internal problem with gridsize!");
 
-  header[4] = xsize;
-  header[5] = ysize;
+  cdi_check_gridsize_int_limit("SERVICE", gridInqSize(gridID));
+
+  header[4] = (int)xsize;
+  header[5] = (int)ysize;
   header[6] = 0;
   header[7] = 0;
 
-  int datatype = record->prec;
-  srvp->dprec = srvDefDatatype(datatype);
+  srvrec_t *srvp = (srvrec_t*) record->exsep;
+  srvp->dprec = srvDefDatatype(record->prec);
 
   srvDefHeader(srvp, header);
 }
 
-
-void srvWriteRecord(stream_t *streamptr, const double *data)
+static
+void srv_write_recordSP(stream_t *streamptr, const float *data)
 {
-  int fileID = streamptr->fileID;
   void *srvp = streamptr->record->exsep;
-
-  srvDefDataDP(srvp, data);
-  srvWrite(fileID, srvp);
+  srvDefDataSP(srvp, data);
+  srvWrite(streamptr->fileID, srvp);
 }
 
 static
-void srv_add_record(stream_t *streamptr, int param, int level, int xsize, int ysize,
+void srv_write_recordDP(stream_t *streamptr, const double *data)
+{
+  void *srvp = streamptr->record->exsep;
+  srvDefDataDP(srvp, data);
+  srvWrite(streamptr->fileID, srvp);
+}
+
+
+void srv_write_record(stream_t *streamptr, int memtype, const void *data)
+{
+  if (memtype == MEMTYPE_DOUBLE)
+    srv_write_recordDP(streamptr, (const double *) data);
+  else
+    srv_write_recordSP(streamptr, (const float *) data);
+}
+
+static
+void srv_add_record(stream_t *streamptr, int param, int level, size_t xsize, size_t ysize,
                     size_t recsize, off_t position, int prec)
 {
-  int vlistID = streamptr->vlistID;
-  int tsID    = streamptr->curTsID;
-  int recID   = recordNewEntry(streamptr, tsID);
+  const int vlistID = streamptr->vlistID;
+  const int tsID    = streamptr->curTsID;
+  const int recID   = recordNewEntry(streamptr, tsID);
   record_t *record = &streamptr->tsteps[tsID].records[recID];
 
   record->size     = recsize;
@@ -194,25 +231,20 @@ void srv_add_record(stream_t *streamptr, int param, int level, int xsize, int ys
   grid->x.size = xsize;
   grid->y.size = ysize;
   struct addIfNewRes gridAdded = cdiVlistAddGridIfNew(vlistID, grid, 0);
-  int gridID = gridAdded.Id;
+  const int gridID = gridAdded.Id;
   if (!gridAdded.isNew) Free(grid);
-  /*
-  if ( level == 0 ) leveltype = ZAXIS_SURFACE;
-  else              leveltype = ZAXIS_GENERIC;
-  */
-  int leveltype = ZAXIS_GENERIC;
 
-  int datatype = srvInqDatatype(prec);
+  const int leveltype = ZAXIS_GENERIC;
+  const int datatype = srvInqDatatype(prec);
 
-  int levelID = 0;
-  int varID;
+  int varID, levelID = 0;
   varAddRecord(recID, param, gridID, leveltype, 0, level, 0, 0, 0,
-	       datatype, &varID, &levelID, TSTEP_INSTANT, 0, 0, -1,
-               NULL, NULL, NULL, NULL, NULL, NULL);
+	       datatype, &varID, &levelID, TSTEP_INSTANT, 0, -1,
+               NULL, NULL, NULL, NULL);
 
   xassert(varID <= SHRT_MAX && levelID <= SHRT_MAX);
   record->varID   = (short)varID;
-  record->levelID = (short)levelID;
+  record->levelID = levelID;
 
   streamptr->tsteps[tsID].nallrecs++;
   streamptr->nrecs++;
@@ -226,58 +258,55 @@ void srvScanTimestep1(stream_t *streamptr)
 {
   DateTime datetime0 = { LONG_MIN, LONG_MIN };
   off_t recpos;
-  taxis_t *taxis;
   srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
   streamptr->curTsID = 0;
 
-  {
-    int tsID  = tstepsNewEntry(streamptr);
-    if ( tsID != 0 )
-      Error("Internal problem! tstepsNewEntry returns %d", tsID);
-    taxis = &streamptr->tsteps[tsID].taxis;
-  }
+  int tsID  = tstepsNewEntry(streamptr);
+  if ( tsID != 0 ) Error("Internal problem! tstepsNewEntry returns %d", tsID);
+  taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
-  int fileID = streamptr->fileID;
+  const int fileID = streamptr->fileID;
 
   int nrecs = 0;
-  while ( TRUE )
+  while ( true )
     {
-      int header[8];
       recpos = fileGetPos(fileID);
-      int status = srvRead(fileID, srvp);
-      if ( status != 0 )
+      if ( srvRead(fileID, srvp) != 0 )
 	{
 	  streamptr->ntsteps = 1;
 	  break;
 	}
-      size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
+      const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
+
+      int header[8];
       srvInqHeader(srvp, header);
 
-      int prec   = srvp->dprec;
-      int rcode  = header[0];
-      int rlevel = header[1];
-      int vdate  = header[2];
-      int vtime  = header[3];
-      int rxsize = header[4];
-      int rysize = header[5];
-
-      int param = cdiEncodeParam(rcode, 255, 255);
+      const int prec   = srvp->dprec;
+      const int rcode  = header[0];
+      const int rlevel = header[1];
+      const int vdate  = header[2];
+      const int vtime  = header[3];
+      const int rxsize = header[4];
+      const int rysize = header[5];
+      const int param  = cdiEncodeParam(rcode, 255, 255);
+      DateTime datetime = { .date = vdate, .time = vtime };
 
       if ( nrecs == 0 )
 	{
-	  datetime0.date = vdate;
-	  datetime0.time = vtime;
+	  datetime0 = datetime;
+          taxis->vdate = vdate;
+          taxis->vtime = vtime;
 	}
       else
 	{
+          record_t *records = streamptr->tsteps[tsID].records;
 	  for ( int recID = 0; recID < nrecs; recID++ )
-            if (    streamptr->tsteps[0].records[recID].param  == param
-                 && streamptr->tsteps[0].records[recID].ilevel == rlevel )
+            if ( param == records[recID].param && rlevel == records[recID].ilevel )
               goto tstepScanLoopFinished;
-	  DateTime datetime = { .date = vdate, .time = vtime };
-	  if ( datetimeCmp(datetime, datetime0) )
+
+	  if ( datetimeDiffer(datetime, datetime0) )
 	    Warning("Inconsistent verification time for code %d level %d", rcode, rlevel);
 	}
 
@@ -294,49 +323,20 @@ void srvScanTimestep1(stream_t *streamptr)
 
   cdi_generate_vars(streamptr);
 
-  int taxisID = taxisCreate(TAXIS_ABSOLUTE);
-  taxis->type  = TAXIS_ABSOLUTE;
-  taxis->vdate = (int)datetime0.date;
-  taxis->vtime = (int)datetime0.time;
+  const int taxisID = taxisCreate(TAXIS_ABSOLUTE);
+  taxis->type = TAXIS_ABSOLUTE;
+  taxis->rdate = taxis->vdate;
+  taxis->rtime = taxis->vtime;
 
-  int vlistID = streamptr->vlistID;
+  const int vlistID = streamptr->vlistID;
   vlistDefTaxis(vlistID, taxisID);
 
   vlist_check_contents(vlistID);
 
-  int nrecords = streamptr->tsteps[0].nallrecs;
-  if ( nrecords < streamptr->tsteps[0].recordSize )
-    {
-      streamptr->tsteps[0].recordSize = nrecords;
-      streamptr->tsteps[0].records =
-	(record_t *) Realloc(streamptr->tsteps[0].records,
-                             (size_t)nrecords * sizeof(record_t));
-    }
+  streamScanResizeRecords1(streamptr);
 
-  streamptr->tsteps[0].recIDs = (int *) Malloc((size_t)nrecords * sizeof (int));
-  streamptr->tsteps[0].nrecs = nrecords;
-  for ( int recID = 0; recID < nrecords; recID++ )
-    streamptr->tsteps[0].recIDs[recID] = recID;
-
-  if ( streamptr->ntsteps == -1 )
-    {
-      int tsID = tstepsNewEntry(streamptr);
-      if ( tsID != streamptr->rtsteps )
-	Error("Internal error. tsID = %d", tsID);
-
-      streamptr->tsteps[tsID-1].next   = TRUE;
-      streamptr->tsteps[tsID].position = recpos;
-    }
-
-  if ( streamptr->ntsteps == 1 )
-    {
-      if ( taxis->vdate == 0 && taxis->vtime == 0 )
-	{
-	  streamptr->ntsteps = 0;
-	  for ( int varID = 0; varID < streamptr->nvars; varID++ )
-            vlistDefVarTsteptype(vlistID, varID, TSTEP_CONSTANT);
-	}
-    }
+  streamScanTsFixNtsteps(streamptr, recpos);
+  streamScanTimeConstAdjust(streamptr, taxis);
 }
 
 static
@@ -344,57 +344,43 @@ int srvScanTimestep2(stream_t *streamptr)
 {
   int header[8];
   off_t recpos = 0;
-  srvcompvar_t compVar, compVar0;
   void *srvp = streamptr->record->exsep;
 
   streamptr->curTsID = 1;
 
-  int vlistID = streamptr->vlistID;
-  int fileID  = streamptr->fileID;
+  const int vlistID = streamptr->vlistID;
+  const int fileID  = streamptr->fileID;
 
-  int tsID = streamptr->rtsteps;
-  if ( tsID != 1 )
-    Error("Internal problem! unexpected timestep %d", tsID+1);
+  const int tsID = streamptr->rtsteps;
+  if ( tsID != 1 ) Error("Internal problem! unexpected timestep %d", tsID+1);
 
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
   fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
   cdi_create_records(streamptr, tsID);
+  record_t *records = streamptr->tsteps[tsID].records;
 
-  int nrecords = streamptr->tsteps[0].nallrecs;
-  streamptr->tsteps[1].recIDs = (int *) Malloc((size_t)nrecords * sizeof (int));
-  streamptr->tsteps[1].nrecs = 0;
-  for ( int recID = 0; recID < nrecords; recID++ )
-    streamptr->tsteps[1].recIDs[recID] = -1;
-
-  for ( int recID = 0; recID < nrecords; recID++ )
-    {
-      streamptr->tsteps[tsID].records[recID].position =
-	streamptr->tsteps[0].records[recID].position;
-      streamptr->tsteps[tsID].records[recID].size     =
-	streamptr->tsteps[0].records[recID].size;
-    }
+  const int nrecords = streamScanInitRecords2(streamptr);
 
   for ( int rindex = 0; rindex <= nrecords; rindex++ )
     {
       recpos = fileGetPos(fileID);
-      int status = srvRead(fileID, srvp);
-      if ( status != 0 )
+      if ( srvRead(fileID, srvp) != 0 )
 	{
 	  streamptr->ntsteps = 2;
 	  break;
 	}
-      size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
+
+      const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
       srvInqHeader(srvp, header);
 
-      int rcode  = header[0];
-      int rlevel = header[1];
-      int vdate  = header[2];
-      int vtime  = header[3];
-
-      int param = cdiEncodeParam(rcode, 255, 255);
+      const int rcode  = header[0];
+      const int rlevel = header[1];
+      const int vdate  = header[2];
+      const int vtime  = header[3];
+      const int param  = cdiEncodeParam(rcode, 255, 255);
 
       if ( rindex == 0 )
 	{
@@ -403,24 +389,19 @@ int srvScanTimestep2(stream_t *streamptr)
 	  taxis->vtime = vtime;
 	}
 
-      compVar.param = param;
-      compVar.level = rlevel;
       bool nextstep = false;
       int recID;
       for ( recID = 0; recID < nrecords; recID++ )
 	{
-	  compVar0.param  = streamptr->tsteps[tsID].records[recID].param;
-	  compVar0.level = streamptr->tsteps[tsID].records[recID].ilevel;
-
-	  if ( memcmp(&compVar0, &compVar, sizeof(srvcompvar_t)) == 0 )
+	  if ( param == records[recID].param && rlevel == records[recID].ilevel )
 	    {
-	      if ( streamptr->tsteps[tsID].records[recID].used )
+	      if ( records[recID].used )
 		{
 		  nextstep = true;
 		}
 	      else
 		{
-		  streamptr->tsteps[tsID].records[recID].used = TRUE;
+		  records[recID].used = true;
 		  streamptr->tsteps[tsID].recIDs[rindex] = recID;
 		}
 	      break;
@@ -437,30 +418,23 @@ int srvScanTimestep2(stream_t *streamptr)
       if ( CDI_Debug )
 	Message("%4d%8d%4d%8d%8d%6d", rindex+1, (int)recpos, rcode, rlevel, vdate, vtime);
 
-      streamptr->tsteps[tsID].records[recID].size = recsize;
-
-      compVar0.param  = streamptr->tsteps[tsID].records[recID].param;
-      compVar0.level = streamptr->tsteps[tsID].records[recID].ilevel;
-
-      if ( memcmp(&compVar0, &compVar, sizeof(srvcompvar_t)) != 0 )
+      if ( param != records[recID].param || rlevel != records[recID].ilevel )
 	{
 	  Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d",
-		  tsID, recID,
-		  streamptr->tsteps[tsID].records[recID].param, param,
-		  streamptr->tsteps[tsID].records[recID].ilevel, rlevel);
+		  tsID, recID, records[recID].param, param, records[recID].ilevel, rlevel);
 	  return CDI_EUFSTRUCT;
 	}
 
-      streamptr->tsteps[1].records[recID].position = recpos;
+      records[recID].position = recpos;
+      records[recID].size = recsize;
     }
 
   int nrecs = 0;
   for ( int recID = 0; recID < nrecords; recID++ )
     {
-      if ( ! streamptr->tsteps[tsID].records[recID].used )
+      if ( ! records[recID].used )
 	{
-	  int varID = streamptr->tsteps[tsID].records[recID].varID;
-          vlistDefVarTsteptype(vlistID, varID, TSTEP_CONSTANT);
+          vlistDefVarTimetype(vlistID, records[recID].varID, TIME_CONSTANT);
 	}
       else
 	{
@@ -471,15 +445,7 @@ int srvScanTimestep2(stream_t *streamptr)
 
   streamptr->rtsteps = 2;
 
-  if ( streamptr->ntsteps == -1 )
-    {
-      tsID = tstepsNewEntry(streamptr);
-      if ( tsID != streamptr->rtsteps )
-	Error("Internal error. tsID = %d", tsID);
-
-      streamptr->tsteps[tsID-1].next   = TRUE;
-      streamptr->tsteps[tsID].position = recpos;
-    }
+  streamScanTsFixNtsteps(streamptr, recpos);
 
   return 0;
 }
@@ -491,11 +457,9 @@ int srvInqContents(stream_t *streamptr)
 
   srvScanTimestep1(streamptr);
 
-  int status = 0;
-  if ( streamptr->ntsteps == -1 ) status = srvScanTimestep2(streamptr);
+  const int status = (streamptr->ntsteps == -1) ? srvScanTimestep2(streamptr) : 0;
 
-  int fileID = streamptr->fileID;
-  fileSetPos(fileID, 0, SEEK_SET);
+  fileSetPos(streamptr->fileID, 0, SEEK_SET);
 
   return status;
 }
@@ -504,64 +468,46 @@ static
 long srvScanTimestep(stream_t *streamptr)
 {
   int header[8];
-  /* int rxsize = 0, rysize = 0; */
   off_t recpos = 0;
-  int recID;
   int nrecs = 0;
   void *srvp = streamptr->record->exsep;
-  /*
-  if ( CDI_Debug )
-    {
-      Message("streamID = %d", streamptr->self);
-      Message("cts = %d", streamptr->curTsID);
-      Message("rts = %d", streamptr->rtsteps);
-      Message("nts = %d", streamptr->ntsteps);
-    }
-  */
 
-  int tsID  = streamptr->rtsteps;
+  int tsID = streamptr->rtsteps;
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
   if ( streamptr->tsteps[tsID].recordSize == 0 )
     {
       cdi_create_records(streamptr, tsID);
+      record_t *records = streamptr->tsteps[tsID].records;
 
-      nrecs = streamptr->tsteps[1].nrecs;
+      nrecs = streamScanInitRecords(streamptr, tsID);
 
-      streamptr->tsteps[tsID].nrecs = nrecs;
-      streamptr->tsteps[tsID].recIDs = (int *) Malloc((size_t)nrecs * sizeof (int));
-      for ( recID = 0; recID < nrecs; recID++ )
-	streamptr->tsteps[tsID].recIDs[recID] = streamptr->tsteps[1].recIDs[recID];
-
-      int fileID = streamptr->fileID;
+      const int fileID = streamptr->fileID;
 
       fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
       for ( int rindex = 0; rindex <= nrecs; rindex++ )
 	{
 	  recpos = fileGetPos(fileID);
-	  int status = srvRead(fileID, srvp);
-	  if ( status != 0 )
+	  if ( srvRead(fileID, srvp) != 0 )
 	    {
 	      streamptr->ntsteps = streamptr->rtsteps + 1;
 	      break;
 	    }
-	  size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
+
+	  const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
 	  srvInqHeader(srvp, header);
 
-	  int rcode  = header[0];
-	  int rlevel = header[1];
-	  int vdate  = header[2];
-	  int vtime  = header[3];
-          /* rxsize = header[4]; */
-          /* rysize = header[5]; */
-
-	  int param = cdiEncodeParam(rcode, 255, 255);
+	  const int rcode  = header[0];
+	  const int rlevel = header[1];
+	  const int vdate  = header[2];
+	  const int vtime  = header[3];
+	  const int param  = cdiEncodeParam(rcode, 255, 255);
 
 	  // if ( rindex == nrecs ) break; gcc-4.5 internal compiler error
 	  if ( rindex == nrecs ) continue;
-	  recID = streamptr->tsteps[tsID].recIDs[rindex];
+	  const int recID = streamptr->tsteps[tsID].recIDs[rindex];
 
 	  if ( rindex == 0 )
 	    {
@@ -570,18 +516,15 @@ long srvScanTimestep(stream_t *streamptr)
 	      taxis->vtime = vtime;
 	    }
 
-          if (    param  != streamptr->tsteps[tsID].records[recID].param
-               || rlevel != streamptr->tsteps[tsID].records[recID].ilevel )
+          if ( param != records[recID].param || rlevel != records[recID].ilevel )
 	    {
 	      Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d",
-		      tsID, recID,
-		      streamptr->tsteps[tsID].records[recID].param, param,
-		      streamptr->tsteps[tsID].records[recID].ilevel, rlevel);
+		      tsID, recID, records[recID].param, param, records[recID].ilevel, rlevel);
 	      Error("Invalid, unsupported or inconsistent record structure!");
 	    }
 
-	  streamptr->tsteps[tsID].records[recID].position = recpos;
-	  streamptr->tsteps[tsID].records[recID].size = recsize;
+	  records[recID].position = recpos;
+	  records[recID].size = recsize;
 
 	  if ( CDI_Debug )
 	    Message("%4d%8d%4d%8d%8d%6d", rindex, (int)recpos, rcode, rlevel, vdate, vtime);
@@ -595,7 +538,7 @@ long srvScanTimestep(stream_t *streamptr)
 	  if ( tsID != streamptr->rtsteps )
 	    Error("Internal error. tsID = %d", tsID);
 
-	  streamptr->tsteps[tsID-1].next   = 1;
+	  streamptr->tsteps[tsID-1].next   = true;
 	  streamptr->tsteps[tsID].position = recpos;
 	}
 
@@ -636,48 +579,43 @@ int srvInqTimestep(stream_t *streamptr, int tsID)
 }
 
 
-void srvReadVarSliceDP(stream_t *streamptr, int varID, int levID, double *data, int *nmiss)
+void srvReadVarSliceDP(stream_t *streamptr, int varID, int levID, double *data, size_t *nmiss)
 {
   if ( CDI_Debug ) Message("streamID = %d  varID = %d  levID = %d", streamptr->self, varID, levID);
 
-  void *srvp = streamptr->record->exsep;
-
   int vlistID  = streamptr->vlistID;
   int fileID   = streamptr->fileID;
-  /* NOTE: tiles are not supported here! */
-  double missval = vlistInqVarMissval(vlistID, varID);
-  int gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
   int tsid     = streamptr->curTsID;
 
   off_t currentfilepos = fileGetPos(fileID);
 
-  /* NOTE: tiles are not supported here! */
   int recID = streamptr->vars[varID].recordTable[0].recordID[levID];
   off_t recpos = streamptr->tsteps[tsid].records[recID].position;
+
   fileSetPos(fileID, recpos, SEEK_SET);
-  if ( srvRead(fileID, srvp) < 0 ) abort();
+
+  void *srvp = streamptr->record->exsep;
+  if (srvRead(fileID, srvp) < 0) abort();
+
   int header[8];
   srvInqHeader(srvp, header);
   srvInqDataDP(srvp, data);
 
   fileSetPos(fileID, currentfilepos, SEEK_SET);
 
-  *nmiss = 0;
-  for ( int i = 0; i < gridsize; i++ )
-    if ( DBL_IS_EQUAL(data[i], missval) || DBL_IS_EQUAL(data[i], (float)missval) )
-      {
-	data[i] = missval;
-	(*nmiss)++;
-      }
+  double missval = vlistInqVarMissval(vlistID, varID);
+  size_t size = gridInqSize(vlistInqVarGrid(vlistID, varID));
+
+  *nmiss = get_num_missvalsDP(size, data, missval);
 }
 
 
-void srvReadVarDP(stream_t *streamptr, int varID, double *data, int *nmiss)
+void srvReadVarDP(stream_t *streamptr, int varID, double *data, size_t *nmiss)
 {
   if ( CDI_Debug ) Message("streamID = %d  varID = %d", streamptr->self, varID);
 
   int vlistID = streamptr->vlistID;
-  size_t gridsize = (size_t) gridInqSize(vlistInqVarGrid(vlistID, varID));
+  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
   size_t nlevs    = (size_t) streamptr->vars[varID].recordTable[0].nlevs;
 
   for ( size_t levID = 0; levID < nlevs; levID++)
@@ -699,21 +637,23 @@ void srvWriteVarSliceDP(stream_t *streamptr, int varID, int levID, const double
 
   int header[8];
   header[0] = pnum;
-  header[1] = (int) zaxisInqLevel(vlistInqVarZaxis(vlistID, varID), levID);
+  header[1] = (int)(zaxisInqLevel(vlistInqVarZaxis(vlistID, varID), levID));
   header[2] = streamptr->tsteps[tsID].taxis.vdate;
   header[3] = streamptr->tsteps[tsID].taxis.vtime;
 
-  int xsize = gridInqXsize(gridID);
-  int ysize = gridInqYsize(gridID);
+  size_t xsize = gridInqXsize(gridID);
+  size_t ysize = gridInqYsize(gridID);
   if ( xsize == 0 || ysize == 0 )
     {
       xsize = gridInqSize(gridID);
       ysize = 1;
     }
   if ( gridInqType(gridID) == GRID_UNSTRUCTURED ) ysize = 1;
-  if ( gridInqSize(gridID) != xsize*ysize )
+  if ( (size_t)gridInqSize(gridID) != xsize*ysize )
     Error("Internal problem with gridsize!");
 
+  cdi_check_gridsize_int_limit("SERVICE", gridInqSize(gridID));
+
   header[4] = xsize;
   header[5] = ysize;
   header[6] = 0;
@@ -735,7 +675,7 @@ void srvWriteVarDP(stream_t *streamptr, int varID, const double *data)
   if ( CDI_Debug ) Message("streamID = %d  varID = %d", streamptr->self, varID);
 
   int vlistID = streamptr->vlistID;
-  size_t gridsize = (size_t) gridInqSize(vlistInqVarGrid(vlistID, varID));
+  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
   size_t nlevs    = (size_t) zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
 
   for ( size_t levID = 0; levID < nlevs; levID++ )
