@@ -25,12 +25,10 @@ compute_curvilinear(double *coords_,
 
 int main(int argc, const char **argv)
 {
-  /* todo: handle optional arguments here to increase test coverage */
-  const char *fname = "test.nc";
-  if (argc > 1)
-    fname = argv[1];
-  int streamID = streamOpenWrite(fname, CDI_FILETYPE_NC);
+  // todo: handle optional arguments here to increase test coverage
+  const char *fname = (argc > 1) ? argv[1] : "test.nc";
 
+  int streamID = streamOpenWrite(fname, CDI_FILETYPE_NC);
   if ( streamID < 0 )
     {
       fprintf(stderr, "Open failed on %s: %s\n", fname,
@@ -78,7 +76,7 @@ int main(int argc, const char **argv)
   int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
 
   int vlistID = vlistCreate();
-  int varID = vlistDefVar(vlistID, gridID, zaxisID, TSTEP_INSTANT);
+  int varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
   vlistDefVarMissval(vlistID, varID, missValue);
   {
     static const char creatorText[] = "CDI test_cdf_write";
