@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -66,6 +56,7 @@
                  2 - reference value
 
 */
+
 struct gaindx
 {
   int type;      /* Indexing file type             */
@@ -78,6 +69,7 @@ struct gaindx
   int *intpnt;   /* Pointer to int index values    */
   float *fltpnt; /* Pointer to float index values  */
 };
+
 struct gaindxb
 {
   int bignum;    /* Number of off_t values */
@@ -89,17 +81,14 @@ struct gaindxb
 static void
 gabswp(void *r, int cnt)
 {
-  int i;
-  char *ch1, *ch2, *ch3, *ch4, cc1, cc2;
-
-  ch1 = (char *) r;
-  ch2 = ch1 + 1;
-  ch3 = ch2 + 1;
-  ch4 = ch3 + 1;
-  for (i = 0; i < cnt; i++)
+  char *ch1 = (char *) r;
+  char *ch2 = ch1 + 1;
+  char *ch3 = ch2 + 1;
+  char *ch4 = ch3 + 1;
+  for (int i = 0; i < cnt; i++)
     {
-      cc1 = *ch1;
-      cc2 = *ch2;
+      char cc1 = *ch1;
+      char cc2 = *ch2;
       *ch1 = *ch4;
       *ch2 = *ch3;
       *ch3 = cc2;
@@ -119,26 +108,17 @@ gabswp(void *r, int cnt)
 static float
 ibm2flt(unsigned char *ibm)
 {
-
-  int positive, power;
-  unsigned int abspower;
-  long int mant;
-  double value, exp;
-
-  positive = (ibm[0] & 0x80) == 0;
-  mant = (ibm[1] << 16) + (ibm[2] << 8) + ibm[3];
-  power = (int) (ibm[0] & 0x7f) - 64;
-  abspower = power > 0 ? power : -power;
+  int positive = (ibm[0] & 0x80) == 0;
+  long mant = (ibm[1] << 16) + (ibm[2] << 8) + ibm[3];
+  int power = (int) (ibm[0] & 0x7f) - 64;
+  unsigned int abspower = power > 0 ? power : -power;
 
   /* calc exp */
-  exp = 16.0;
-  value = 1.0;
+  double exp = 16.0;
+  double value = 1.0;
   while (abspower)
     {
-      if (abspower & 1)
-        {
-          value *= exp;
-        }
+      if (abspower & 1) value *= exp;
       exp = exp * exp;
       abspower >>= 1;
     }
@@ -146,6 +126,7 @@ ibm2flt(unsigned char *ibm)
   if (power < 0) value = 1.0 / value;
   value = value * mant / 16777216.0;
   if (positive == 0) value = -value;
+
   return (float) value;
 }
 
@@ -159,9 +140,7 @@ ibm2flt(unsigned char *ibm)
 static int
 flt2ibm(float x, unsigned char *ibm)
 {
-
   int sign, exp, i;
-  double mant;
 
   if (std::fabs((double) x) <= 0)
     {
@@ -178,7 +157,7 @@ flt2ibm(float x, unsigned char *ibm)
   else
     sign = 0;
 
-  mant = frexp((double) x, &exp);
+  double mant = frexp((double) x, &exp);
 
   /* round up by adding 2**-24 */
   /* mant = mant + 1.0/16777216.0; */
@@ -249,7 +228,6 @@ dumpmap()
   struct gaindx indx;
   struct gaindxb indxb;
   size_t nbytes;
-  FILE *mapfp;
 
   indx.hipnt = nullptr;
   indx.hfpnt = nullptr;
@@ -258,8 +236,8 @@ dumpmap()
   indxb.bigpnt = nullptr;
   indxb.bignum = 0;
 
-  mapfp = fopen(cdoGetStreamName(0), "r");
-  if (mapfp == nullptr) cdoAbort("Open failed on %s", cdoGetStreamName(0));
+  auto mapfp = fopen(cdo_get_stream_name(0), "r");
+  if (mapfp == nullptr) cdo_abort("Open failed on %s", cdo_get_stream_name(0));
 
   /* check the version number */
 
@@ -369,7 +347,7 @@ dumpmap()
         }
     }
 
-  (void)(nbytes); //CDO_UNUSED
+  (void) (nbytes);  // CDO_UNUSED
 
   fclose(mapfp);
 
@@ -417,7 +395,6 @@ static void
 ctl_xydef(FILE *gdp, int gridID, bool *yrev)
 {
   int i, j;
-  double xfirst, yfirst, xinc, yinc;
 
   *yrev = false;
 
@@ -436,17 +413,17 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
       int nx = 0, ny = 0, ni;
       double inc[] = { 1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001 };
 
-      double xinc = gridInqXinc(gridID);
-      double yinc = gridInqYinc(gridID);
+      auto xinc = gridInqXinc(gridID);
+      auto yinc = gridInqYinc(gridID);
 
-      double lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0;
+      struct CDI_GridProjParams gpp;
+      gridInqParamsLCC(gridID, &gpp);
 
-      gridInqParamLCC(gridID, cdiInqGridMissval(), &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
-      fprintf(gdp, "PDEF %d %d LCCR %g %g 1 1 %g %g %g %g %g\n", xsize, ysize, xval_0, yval_0, lat_1, lat_2, lon_0, xinc, yinc);
+      fprintf(gdp, "PDEF %d %d LCCR %g %g 1 1 %g %g %g %g %g\n", xsize, ysize, gpp.xval_0, gpp.yval_0, gpp.lat_1, gpp.lat_2,
+              gpp.lon_0, xinc, yinc);
 
       gridID = gridToCurvilinear(gridID, 0);
-      Varray<double> xvals(xsize * ysize);
-      Varray<double> yvals(xsize * ysize);
+      Varray<double> xvals(xsize * ysize), yvals(xsize * ysize);
       gridInqXvals(gridID, xvals.data());
       gridInqYvals(gridID, yvals.data());
       for (i = 0; i < xsize * ysize; ++i)
@@ -458,8 +435,8 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
           if (yvals[i] > ymax) ymax = yvals[i];
         }
 
-      xfirst = ((int) (xmin - 0.0));
-      yfirst = ((int) (ymin - 0.0));
+      double xfirst = ((int) (xmin - 0.0));
+      double yfirst = ((int) (ymin - 0.0));
       xrange = ((int) (xmax + 1.5)) - xfirst;
       yrange = ((int) (ymax + 1.5)) - yfirst;
 
@@ -481,8 +458,8 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
     }
   else
     {
-      xfirst = gridInqXval(gridID, 0);
-      xinc = gridInqXinc(gridID);
+      auto xfirst = gridInqXval(gridID, 0);
+      auto xinc = gridInqXinc(gridID);
       if (IS_EQUAL(xinc, 0) && gridInqXvals(gridID, nullptr))
         {
           Varray<double> xvals(xsize);
@@ -513,8 +490,8 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
 
   if (!(gridtype == GRID_PROJECTION && projtype == CDI_PROJ_LCC))
     {
-      yfirst = gridInqYval(gridID, 0);
-      yinc = gridInqYinc(gridID);
+      auto yfirst = gridInqYval(gridID, 0);
+      auto yinc = gridInqYinc(gridID);
       if (gridtype == GRID_GAUSSIAN) yinc = 0;
 
       if (IS_EQUAL(yinc, 0) && gridInqYvals(gridID, nullptr))
@@ -593,7 +570,7 @@ ctl_zdef(FILE *gdp, int vlistID, bool *zrev)
     }
 
   Varray<double> levels(nlevmax);
-  cdoZaxisInqLevels(zaxisIDmax, levels.data());
+  cdo_zaxis_inq_levels(zaxisIDmax, levels.data());
   bool lplev = (zaxisInqType(zaxisIDmax) == ZAXIS_PRESSURE);
   double level0 = levels[0];
   if (nlevmax > 1)
@@ -702,7 +679,8 @@ ctl_vars(FILE *gdp, int filetype, int vlistID, int nvarsout, int *vars)
       if (vars[varID] == true)
         {
           zaxisID = vlistInqVarZaxis(vlistID, varID);
-          ltype = zaxisInqLtype(zaxisID);
+          ltype = 0;
+          cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
           nlev = zaxisInqSize(zaxisID);
           vlistInqVarName(vlistID, varID, varname);
 
@@ -789,13 +767,12 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
   int i;
   struct gaindx indx;
   struct gaindxb indxb;
-  FILE *mapfp;
   int hinum[5];
 
   memset(&indx, 0, sizeof(struct gaindx));
 
-  mapfp = fopen(ctlfile, "w");
-  if (mapfp == nullptr) cdoAbort("Open failed on %s", ctlfile);
+  auto mapfp = fopen(ctlfile, "w");
+  if (mapfp == nullptr) cdo_abort("Open failed on %s", ctlfile);
 
   indx.type = map_version;
   indx.hfnum = 0;
@@ -878,7 +855,7 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
         {
           fdum = fltnum[i];
           rc = flt2ibm(fdum, ibmfloat);
-          if (rc < 0) cdoAbort("overflow in IBM float conversion");
+          if (rc < 0) cdo_abort("overflow in IBM float conversion");
           for (j = 0; j < 4; j++) map[bcnt++] = ibmfloat[j];
         }
 
@@ -888,7 +865,7 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
         {
           fdum = 0;
           rc = flt2ibm(fdum, ibmfloat);
-          if (rc < 0) cdoAbort("overflow in IBM float conversion");
+          if (rc < 0) cdo_abort("overflow in IBM float conversion");
           for (j = 0; j < 4; j++) map[bcnt++] = ibmfloat[j];
         }
 
@@ -937,9 +914,6 @@ Gradsdes(void *process)
   int gridID = -1;
   int gridtype = -1;
   int index;
-  int varID;
-  int levelID;
-  int nrecs;
   char *idxfile = nullptr;
   char varname[CDI_MAX_NAME];
   bool yrev = false;
@@ -957,7 +931,6 @@ Gradsdes(void *process)
   int idmn, idhh, idmm, idyy, iddd;
   int dt = 1, iik = 0, mdt = 0;
   size_t gridsize = 0;
-  size_t nmiss;
   int prec;
   int map_version = 2;
   int maxrecs = 0;
@@ -968,18 +941,18 @@ Gradsdes(void *process)
   Varray<double> array;
   const char *cmons[] = { "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" };
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   // clang-format off
-  const auto GRADSDES = cdoOperatorAdd("gradsdes",  0, 0, nullptr);
-  const auto DUMPMAP  = cdoOperatorAdd("dumpmap",   0, 0, nullptr);
+  const auto GRADSDES = cdo_operator_add("gradsdes",  0, 0, nullptr);
+  const auto DUMPMAP  = cdo_operator_add("dumpmap",   0, 0, nullptr);
   // clang-format on
 
-  (void)(GRADSDES); //CDO_UNUSED
+  (void) (GRADSDES);  // CDO_UNUSED
 
-  const auto operatorID = cdoOperatorID();
+  const auto operatorID = cdo_operator_id();
 
-  auto datfile = cdoGetStreamName(0);
+  auto datfile = cdo_get_stream_name(0);
   const auto len = strlen(datfile);
   char *ctlfile = (char *) Malloc(len + 10);
   strcpy(ctlfile, datfile);
@@ -987,37 +960,37 @@ Gradsdes(void *process)
   if (operatorID == DUMPMAP)
     {
       dumpmap();
-      cdoFinish();
+      cdo_finish();
       return 0;
     }
 
-  if (operatorArgc() > 1) cdoAbort("Too many arguments!");
+  if (cdo_operator_argc() > 1) cdo_abort("Too many arguments!");
 
-  if (operatorArgc() == 1)
+  if (cdo_operator_argc() == 1)
     {
-      map_version = parameter2int(cdoOperatorArgv(0));
-      if (map_version != 1 && map_version != 2 && map_version != 4) cdoAbort("map_version=%d unsupported!", map_version);
+      map_version = parameter_to_int(cdo_operator_argv(0));
+      if (map_version != 1 && map_version != 2 && map_version != 4) cdo_abort("map_version=%d unsupported!", map_version);
     }
   else
     {
-      if (fileSize(cdoGetStreamName(0)) > 2147483647L) map_version = 4;
+      if (FileUtils::size(cdo_get_stream_name(0)) > 2147483647L) map_version = 4;
     }
 
-  if (Options::cdoVerbose) cdoPrint("GrADS GRIB map version: %d", map_version);
+  if (Options::cdoVerbose) cdo_print("GrADS GRIB map version: %d", map_version);
 
   if (map_version == 4 && sizeof(off_t) != 8)
-    cdoAbort("GrADS GRIB map version %d requires size of off_t to be 8! The size of off_t is %ld.", map_version, sizeof(off_t));
+    cdo_abort("GrADS GRIB map version %d requires size of off_t to be 8! The size of off_t is %ld.", map_version, sizeof(off_t));
 
-  const auto streamID = cdoOpenRead(0);
+  const auto streamID = cdo_open_read(0);
 
-  const auto vlistID = cdoStreamInqVlist(streamID);
+  const auto vlistID = cdo_stream_inq_vlist(streamID);
 
   const auto nvars = vlistNvars(vlistID);
   const auto ntsteps = vlistNtsteps(vlistID);
   const auto ngrids = vlistNgrids(vlistID);
 
-  auto filetype = cdoInqFiletype(streamID);
-  auto byteorder = cdoInqByteorder(streamID);
+  auto filetype = cdo_inq_filetype(streamID);
+  auto byteorder = cdo_inq_byteorder(streamID);
 
   if (filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C || filetype == CDI_FILETYPE_NC5)
     filetype = CDI_FILETYPE_NC;
@@ -1025,13 +998,13 @@ Gradsdes(void *process)
   if (filetype != CDI_FILETYPE_SRV && filetype != CDI_FILETYPE_EXT && filetype != CDI_FILETYPE_IEG && filetype != CDI_FILETYPE_GRB)
     {
       if (filetype == CDI_FILETYPE_NC)
-        //        cdoAbort("Unsupported file format: NetCDF");
+        //        cdo_abort("Unsupported file format: NetCDF");
         ;
       else if (filetype == CDI_FILETYPE_GRB2)
-        // cdoAbort("Unsupported file format: GRIB2");
+        // cdo_abort("Unsupported file format: GRIB2");
         ;
       else
-        cdoAbort("Unsupported file format!");
+        cdo_abort("Unsupported file format!");
     }
 
   /* find the first lonlat or Gaussian grid */
@@ -1045,14 +1018,13 @@ Gradsdes(void *process)
         break;
     }
 
-  if (index == ngrids) cdoAbort("No Lon/Lat, Gaussian or Lambert grid found (%s data unsupported)!", gridNamePtr(gridtype));
+  if (index == ngrids) cdo_abort("No Lon/Lat, Gaussian or Lambert grid found (%s data unsupported)!", gridNamePtr(gridtype));
 
   /* select all variables with used gridID */
   std::vector<int> vars(nvars);
   std::vector<int> recoffset(nvars);
-  int nvarsout = 0;
-  int nrecsout = 0;
-  for (varID = 0; varID < nvars; varID++)
+  int nvarsout = 0, nrecsout = 0;
+  for (int varID = 0; varID < nvars; varID++)
     {
       if (vlistInqVarGrid(vlistID, varID) == gridID)
         {
@@ -1066,18 +1038,18 @@ Gradsdes(void *process)
           nvarsout++;
           nrecsout += zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
           if (ntsteps != 1 && ntsteps != 0 && vlistInqVarTimetype(vlistID, varID) == TIME_CONSTANT)
-            cdoAbort("Unsupported GrADS record structure! Variable %d has only 1 time step.", vlistInqVarCode(vlistID, varID));
+            cdo_abort("Unsupported GrADS record structure! Variable %d has only 1 time step.", vlistInqVarCode(vlistID, varID));
         }
       else
         {
           vlistInqVarName(vlistID, varID, varname);
-          cdoPrint("Unsupported grid type >%s<, skipped variable %s!", gridNamePtr(gridInqType(vlistInqVarGrid(vlistID, varID))),
-                   varname);
+          cdo_print("Unsupported grid type >%s<, skipped variable %s!", gridNamePtr(gridInqType(vlistInqVarGrid(vlistID, varID))),
+                    varname);
           vars[varID] = false;
         }
     }
 
-  if (filetype != CDI_FILETYPE_GRB && nvars != nvarsout) cdoAbort("Too many different grids!");
+  if (filetype != CDI_FILETYPE_GRB && nvars != nvarsout) cdo_abort("Too many different grids!");
 
   if (filetype == CDI_FILETYPE_SRV)
     {
@@ -1111,7 +1083,7 @@ Gradsdes(void *process)
 
   /* open ctl file*/
   auto gdp = fopen(ctlfile, "w");
-  if (gdp == nullptr) cdoAbort("Open failed on %s", ctlfile);
+  if (gdp == nullptr) cdo_abort("Open failed on %s", ctlfile);
 
   /* VERSION */
   fprintf(gdp, "* Generated by CDO operator gradsdes\n");
@@ -1124,7 +1096,7 @@ Gradsdes(void *process)
     {
       datfile = strrchr(datfile, '/');
       if (datfile == 0)
-        datfile = cdoGetStreamName(0);
+        datfile = cdo_get_stream_name(0);
       else
         datfile++;
       fprintf(gdp, "DSET  ^%s\n", datfile);
@@ -1183,8 +1155,11 @@ Gradsdes(void *process)
   bool cal365day = (taxisInqCalendar(taxisID) == CALENDAR_365DAYS);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
+  while (true)
     {
+      const auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
+      if (nrecs == 0) break;
+
       const auto vdate = taxisInqVdate(taxisID);
       const auto vtime = taxisInqVtime(taxisID);
 
@@ -1288,14 +1263,16 @@ Gradsdes(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              cdoInqRecord(streamID, &varID, &levelID);
+              int varID, levelID;
+              cdo_inq_record(streamID, &varID, &levelID);
               if (vars[varID] == true)
                 {
-                  cdoReadRecord(streamID, array.data(), &nmiss);
+                  size_t nmiss;
+                  cdo_read_record(streamID, array.data(), &nmiss);
 
                   index = (tsID * nrecsout + recoffset[varID] + levelID);
 
-                  cdoInqGRIBinfo(streamID, &intnum[index], &fltnum[index * 3], &bignum[index * 2]);
+                  cdo_inq_grib_info(streamID, &intnum[index], &fltnum[index * 3], &bignum[index * 2]);
 
                   if (map_version != 4)
                     {
@@ -1303,9 +1280,9 @@ Gradsdes(void *process)
                       if (checksize < 0L || checksize > 2147483647L)
                         {
                           nrecords -= nrecsout;
-                          cdoWarning("File size limit reached for GrADS GRIB map_version=%d! Only the first %d time "
-                                     "steps (2GB) are processed.",
-                                     map_version, tsID);
+                          cdo_warning("File size limit reached for GrADS GRIB map_version=%d! Only the first %d time "
+                                      "steps (2GB) are processed.",
+                                      map_version, tsID);
                           goto LABEL_STOP;
                         }
                     }
@@ -1334,7 +1311,7 @@ LABEL_STOP:
         {
           /* int iddx = idd0; */
           idd0 = 1;
-          cdoPrint("Reset start date to %02d:%02dZ%02d%s%04d", ihh0, imn0, idd0, cmons[imm0 - 1], iyy0);
+          cdo_print("Reset start date to %02d:%02dZ%02d%s%04d", ihh0, imn0, idd0, cmons[imm0 - 1], iyy0);
         }
     }
 
@@ -1372,18 +1349,18 @@ LABEL_STOP:
     }
   if (filetype == CDI_FILETYPE_GRB2)
     {
-      cdoAbort("The fileformat GRIB2 is not fully supported yet for the gradsdes operator.\n"
-               "The .ctl file %s was generated. You can add the necessary .idx file by running\n\tgribmap -i %s",
-               ctlfile, ctlfile);
+      cdo_abort("The fileformat GRIB2 is not fully supported yet for the gradsdes operator.\n"
+                "The .ctl file %s was generated. You can add the necessary .idx file by running\n\tgribmap -i %s",
+                ctlfile, ctlfile);
       // write_map_grib2(idxfile, map_version, nrecords, intnum, fltnum, bignum);
     }
 
-  cdoStreamClose(streamID);
+  cdo_stream_close(streamID);
 
   if (ctlfile) Free(ctlfile);
   if (idxfile) Free(idxfile);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
