@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -24,7 +14,6 @@
 #include <cdi.h>
 
 #include "cdo_options.h"
-#include "functs.h"
 #include "process_int.h"
 #include "cdo_vlist.h"
 #include "cdo_zaxis.h"
@@ -34,12 +23,12 @@
 #include "progress.h"
 #include "cdi_lockedIO.h"
 
-double datestrToDouble(const char *datestr, int opt);
+double date_str_to_double(const char *datestr, int opt);
 
 int cdo_read_timestepmask(const char *maskfile, std::vector<bool> &imask);
 
 std::string
-domToString(int64_t date)
+dom_to_string(int64_t date)
 {
   int year, month, day;
   cdiDecodeDate(date, &year, &month, &day);
@@ -65,9 +54,9 @@ write_const_vars(CdoStreamID streamID2, const VarList &varList2, const int nvars
           for (int levelID2 = 0; levelID2 < nlevels; ++levelID2)
             {
               auto pdata = &vardata2[varID2][gridsize * levelID2];
-              const auto nmiss = arrayNumMV(gridsize, pdata, missval);
-              cdoDefRecord(streamID2, varID2, levelID2);
-              cdoWriteRecord(streamID2, pdata, nmiss);
+              const auto nmiss = array_num_mv(gridsize, pdata, missval);
+              cdo_def_record(streamID2, varID2, levelID2);
+              cdo_write_record(streamID2, pdata, nmiss);
             }
           vardata2[varID2].clear();
           vardata2[varID2].shrink_to_fit();
@@ -79,14 +68,14 @@ static void
 eval_timestepmask(const char *maskfile, KVList &kvlist)
 {
   std::vector<bool> imask;
-  auto n = cdo_read_timestepmask(maskfile, imask);
-  if (n < 0) cdoAbort("Read of timestep mask failed!");
+  const auto n = cdo_read_timestepmask(maskfile, imask);
+  if (n < 0) cdo_abort("Read of timestep mask failed!");
 
   int nvalues = 0;
   for (int i = 0; i < n; ++i)
     if (imask[i]) nvalues++;
   if (nvalues == 0)
-    cdoPrint("timestepmask has no values!");
+    cdo_print("timestepmask has no values!");
   else
     {
       KeyValues kv;
@@ -97,15 +86,13 @@ eval_timestepmask(const char *maskfile, KVList &kvlist)
       std::vector<char> value;
       int j = 0;
       for (int i = 0; i < n; ++i)
-        {
-          if (imask[i])
-            {
-              const auto length = (size_t) std::log10(j + 1) + 2;
-              value.resize(length);
-              sprintf(value.data(), "%d", i + 1);
-              kv.values[j++] = value.data();
-            }
-        }
+        if (imask[i])
+          {
+            const auto length = (size_t) std::log10(j + 1) + 2;
+            value.resize(length);
+            sprintf(value.data(), "%d", i + 1);
+            kv.values[j++] = value.data();
+          }
 
       kvlist.push_back(kv);
     }
@@ -129,12 +116,21 @@ steptypeName(const int tsteptype)
   return "unknown";
 }
 
+static bool
+has_selected_params(int nvars, const VarList &varList, int vlistID)
+{
+  for (int varID = 0; varID < nvars; ++varID)
+    for (int levID = 0; levID < varList[varID].nlevels; ++levID)
+      if (vlistInqFlag(vlistID, varID, levID) == true) return true;
+
+  return false;
+}
+
 void *
 Select(void *process)
 {
-  bool lconstvars = true;
+  auto hasConstVars = true;
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
-  int nrecs;
   int nvars, nvars2 = 0;
   int varID, levelID;
   int last_year = -999999999;
@@ -144,38 +140,38 @@ Select(void *process)
   int vlistID0 = -1, vlistID2 = -1;
   int taxisID2 = CDI_UNDEFID;
   int ntsteps2 = 0;
-  bool ltimsel = false;
+  auto doTimeSel = false;
   std::vector<bool> vars;
   Varray2D<double> vardata2;
-  Varray<double> array;
-  double fstartdate = -99999999999.;
-  double fenddate = -99999999999.;
+  Field field;
+  double fstartdate = -99999999999.0;
+  double fenddate = -99999999999.0;
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
-  const auto SELECT = cdoOperatorAdd("select", 0, 0, "parameter list");
-  const auto DELETE = cdoOperatorAdd("delete", 0, 0, "parameter list");
+  const auto SELECT = cdo_operator_add("select", 0, 0, "parameter list");
+  const auto DELETE = cdo_operator_add("delete", 0, 0, "parameter list");
 
-  const auto lcopy = unchangedRecord();
+  const auto dataIsUnchanged = data_is_unchanged();
 
-  const auto operatorID = cdoOperatorID();
+  const auto operatorID = cdo_operator_id();
 
-  operatorInputArg(cdoOperatorEnter(operatorID));
+  operator_input_arg(cdo_operator_enter(operatorID));
 
-  const auto argc = operatorArgc();
-  auto argnames = cdoGetOperArgv();
+  const auto argc = cdo_operator_argc();
+  auto argnames = cdo_get_oper_argv();
 
-  if (argc == 0) cdoAbort("Parameter missing!");
+  if (argc == 0) cdo_abort("Parameter missing!");
 
   KVList kvlist;
   kvlist.name = "SELECT";
-  if (kvlist.parseArguments(argc, argnames) != 0) cdoAbort("Parse error!");
+  if (kvlist.parse_arguments(argc, argnames) != 0) cdo_abort("Parse error!");
   if (Options::cdoVerbose) kvlist.print();
 
   auto kv = kvlist.search("timestepmask");
   if (kv && kv->nvalues > 0)
     {
-      if (kvlist.search("timestep")) cdoAbort("Parameter timestep and timestepmask can't be combined!");
+      if (kvlist.search("timestep")) cdo_abort("Parameter timestep and timestepmask can't be combined!");
       eval_timestepmask(kv->values[0].c_str(), kvlist);
     }
 
@@ -195,6 +191,7 @@ Select(void *process)
   SELINFO_ADD_INT(zaxisnum,         "Zaxis number");
   SELINFO_ADD_INT(gridnum,          "Grid number");
   SELINFO_ADD_FLT(level,            "Level");
+  SELINFO_ADD_FLT(levrange,         "Level range");
   SELINFO_ADD_WORD(name,            "Variable name");
   SELINFO_ADD_WORD(param,           "Parameter");
   SELINFO_ADD_WORD(zaxisname,       "Zaxis name");
@@ -212,10 +209,12 @@ Select(void *process)
 
   selInfo.verify();
 
-  if (SELINFO_NVAL(timestepmask) > 1) cdoAbort("Key timestepmask has too many values!");
+  if (SELINFO_NVAL(levrange) > 0 && SELINFO_NVAL(levrange) != 2) cdo_abort("Key levrange needs two values!");
+  if (SELINFO_NVAL(timestepmask) > 1) cdo_abort("Key timestepmask has too many values!");
+  (void) (levrange);      // CDO_UNUSED
   (void) (timestepmask);  // CDO_UNUSED
 
-  const auto nfiles = cdoStreamCnt() - 1;
+  const auto nfiles = cdo_stream_cnt() - 1;
 
   DateTimeList dtlist;
 
@@ -228,21 +227,21 @@ Select(void *process)
   for (int indf = 0; indf < nfiles; ++indf)
     {
       if (!Options::cdoVerbose && nfiles > 1) progress::update(0, 1, (indf + 1.) / nfiles);
-      if (Options::cdoVerbose) cdoPrint("Process file: %s", cdoGetStreamName(indf));
+      if (Options::cdoVerbose) cdo_print("Process file: %s", cdo_get_stream_name(indf));
 
-      const auto streamID1 = cdoOpenRead(indf);
+      const auto streamID1 = cdo_open_read(indf);
 
-      const auto vlistID1 = cdoStreamInqVlist(streamID1);
+      const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
       const auto taxisID1 = vlistInqTaxis(vlistID1);
 
       VarList varList1, varList2;
       varListInit(varList1, vlistID1);
 
-      bool lcopy_const = false;
+      auto copyConstVars = false;
 
       if (indf == 0)
         {
-          bool xresult = true;
+          auto xresult = true;
 
           // vlistID0 = vlistDuplicate(vlistID1);
 
@@ -255,19 +254,18 @@ Select(void *process)
               xresult = false;
               for (varID = 0; varID < nvars; ++varID)
                 {
-                  const auto nlevels = varList1[varID].nlevels;
-                  for (int levID = 0; levID < nlevels; ++levID) vlistDefFlag(vlistID1, varID, levID, true);
+                  for (int levID = 0; levID < varList1[varID].nlevels; ++levID) vlistDefFlag(vlistID1, varID, levID, true);
                 }
             }
 
-          const bool findVariable = SELINFO_NVAL(code) || SELINFO_NVAL(name) || SELINFO_NVAL(param);
+          const auto findVariable = SELINFO_NVAL(code) || SELINFO_NVAL(name) || SELINFO_NVAL(param);
 
-          bool lvarsel = findVariable || SELINFO_NVAL(ltype) || SELINFO_NVAL(zaxisnum) || SELINFO_NVAL(gridnum)
-                         || SELINFO_NVAL(zaxisname) || SELINFO_NVAL(gridname) || SELINFO_NVAL(steptype);
+          auto doVarSel = findVariable || SELINFO_NVAL(ltype) || SELINFO_NVAL(zaxisnum) || SELINFO_NVAL(gridnum)
+                          || SELINFO_NVAL(zaxisname) || SELINFO_NVAL(gridname) || SELINFO_NVAL(steptype);
 
-          bool llevsel = SELINFO_NVAL(level) || SELINFO_NVAL(levidx);
+          auto doLevSel = SELINFO_NVAL(level) || SELINFO_NVAL(levrange) || SELINFO_NVAL(levidx);
 
-          ltimsel = SELINFO_NVAL(date) || SELINFO_NVAL(startdate) || SELINFO_NVAL(enddate) || SELINFO_NVAL(season)
+          doTimeSel = SELINFO_NVAL(date) || SELINFO_NVAL(startdate) || SELINFO_NVAL(enddate) || SELINFO_NVAL(season)
                     || SELINFO_NVAL(timestep_of_year) || SELINFO_NVAL(timestep) || SELINFO_NVAL(year) || SELINFO_NVAL(month)
                     || SELINFO_NVAL(day) || SELINFO_NVAL(hour) || SELINFO_NVAL(minute) || SELINFO_NVAL(dom);
 
@@ -282,19 +280,15 @@ Select(void *process)
               // stdname = varList1[varID].stdname;
               param = paramstr;
 
-              const auto gridID = varList1[varID].gridID;
               const auto zaxisID = varList1[varID].zaxisID;
-              const auto nlevels = varList1[varID].nlevels;
-              ltype = zaxis2ltype(zaxisID);
+              ltype = zaxis_to_ltype(zaxisID);
 
               zaxisnum = vlistZaxisIndex(vlistID1, zaxisID) + 1;
-              const auto zaxistype = zaxisInqType(zaxisID);
-              zaxisName(zaxistype, zname);
+              zaxisName(zaxisInqType(zaxisID), zname);
               zaxisname = zname;
 
-              gridnum = vlistGridIndex(vlistID1, gridID) + 1;
-              const auto gridtype = gridInqType(gridID);
-              gridName(gridtype, gname);
+              gridnum = vlistGridIndex(vlistID1, varList1[varID].gridID) + 1;
+              gridName(gridInqType(varList1[varID].gridID), gname);
               gridname = gname;
 
               steptype = steptypeName(varList1[varID].tsteptype);
@@ -311,7 +305,7 @@ Select(void *process)
               const auto found_zaxis = SELINFO_CHECK(zaxisnum);
               const auto found_zname = SELINFO_CHECK(zaxisname);
 
-              bool lvar = found_code || found_name || found_param;
+              bool lvar = (found_code || found_name || found_param);
               bool lstep = SELINFO_NVAL(steptype) ? found_stype : true;
               bool lgrid = (SELINFO_NVAL(gridnum) || SELINFO_NVAL(gridname)) ? (found_grid || found_gname) : true;
               bool lvert = (SELINFO_NVAL(ltype) || SELINFO_NVAL(zaxisnum) || SELINFO_NVAL(zaxisname))
@@ -328,14 +322,16 @@ Select(void *process)
                     vars[varID] = true;
                   else if (found_ltype || found_zaxis || found_zname)
                     vars[varID] = true;
-                  else if (!lvarsel && (SELINFO_NVAL(levidx) || SELINFO_NVAL(level)))
+                  else if (!doVarSel && (SELINFO_NVAL(levidx) || SELINFO_NVAL(level) || SELINFO_NVAL(levrange)))
                     {
+                      const auto nlevels = varList1[varID].nlevels;
                       for (int levID = 0; levID < nlevels; ++levID)
                         {
                           levidx = levID + 1;
-                          level = cdoZaxisInqLevel(zaxisID, levID);
+                          level = cdo_zaxis_inq_level(zaxisID, levID);
                           if (!vars[varID] && SELINFO_CHECK(levidx)) vars[varID] = true;
                           if (!vars[varID] && SELINFO_CHECK(level)) vars[varID] = true;
+                          if (!vars[varID] && SELINFO_CHECK_RANGE(levrange, level)) vars[varID] = true;
                         }
                     }
                 }
@@ -363,7 +359,7 @@ Select(void *process)
                   for (int levID = 0; levID < nlevels; ++levID)
                     {
                       levidx = levID + 1;
-                      level = cdoZaxisInqLevel(zaxisID, levID);
+                      level = cdo_zaxis_inq_level(zaxisID, levID);
 
                       if (nlevels == 1 && IS_EQUAL(level, 0))
                         {
@@ -380,6 +376,10 @@ Select(void *process)
                             {
                               if (SELINFO_CHECK(level)) vlistDefFlag(vlistID1, varID, levID, xresult);
                             }
+                          else if (SELINFO_NVAL(levrange))
+                            {
+                              if (SELINFO_CHECK_RANGE(levrange, level)) vlistDefFlag(vlistID1, varID, levID, xresult);
+                            }
                           else
                             {
                               vlistDefFlag(vlistID1, varID, levID, xresult);
@@ -400,24 +400,27 @@ Select(void *process)
           SELINFO_CHECK_FLAG(zaxisname);
           SELINFO_CHECK_FLAG(gridname);
           SELINFO_CHECK_FLAG(steptype);
+          SELINFO_CHECK_RANGE_FLAG(levrange);
 
-          int npar = 0;
-          for (varID = 0; varID < nvars; ++varID)
+          if (has_selected_params(nvars, varList1, vlistID1))
             {
-              const auto nlevels = varList1[varID].nlevels;
-              for (int levID = 0; levID < nlevels; ++levID)
-                if (vlistInqFlag(vlistID1, varID, levID) == true)
-                  {
-                    npar++;
-                    break;
-                  }
-            }
-
-          if (npar == 0)
-            {
-              if ((!lvarsel) && (!llevsel) && ltimsel)
+              if (doVarSel && doTimeSel)
                 {
-                  lcopy_const = true;
+                  for (varID = 0; varID < nvars; ++varID)
+                    {
+                      if (vars[varID] == true && varList1[varID].timetype == TIME_CONSTANT)
+                        {
+                          copyConstVars = true;
+                          break;
+                        }
+                    }
+                }
+            }
+          else
+            {
+              if ((!doVarSel) && (!doLevSel) && doTimeSel)
+                {
+                  copyConstVars = true;
 
                   for (varID = 0; varID < nvars; ++varID)
                     {
@@ -428,42 +431,27 @@ Select(void *process)
                 }
               else
                 {
-                  cdoAbort("No variable selected!");
-                }
-            }
-          else
-            {
-              if (lvarsel && ltimsel)
-                {
-                  for (varID = 0; varID < nvars; ++varID)
-                    {
-                      if (vars[varID] == true && varList1[varID].timetype == TIME_CONSTANT)
-                        {
-                          lcopy_const = true;
-                          break;
-                        }
-                    }
+                  cdo_abort("No variable selected!");
                 }
             }
 
-          // if ( Options::cdoVerbose ) vlistPrint(vlistID1);
+          // if (Options::cdoVerbose) vlistPrint(vlistID1);
 
           vlistID0 = vlistDuplicate(vlistID1);
           for (varID = 0; varID < nvars; ++varID)
             {
-              const auto nlevels = varList1[varID].nlevels;
-              for (int levID = 0; levID < nlevels; ++levID)
+              for (int levID = 0; levID < varList1[varID].nlevels; ++levID)
                 vlistDefFlag(vlistID0, varID, levID, vlistInqFlag(vlistID1, varID, levID));
             }
 
-          // if ( Options::cdoVerbose ) vlistPrint(vlistID0);
+          // if (Options::cdoVerbose) vlistPrint(vlistID0);
 
           vlistID2 = vlistCreate();
-          cdoVlistCopyFlag(vlistID2, vlistID0);
+          cdo_vlist_copy_flag(vlistID2, vlistID0);
 
           varListInit(varList2, vlistID2);
 
-          // if ( Options::cdoVerbose ) vlistPrint(vlistID2);
+          // if (Options::cdoVerbose) vlistPrint(vlistID2);
 
           taxisID2 = taxisDuplicate(taxisID1);
 
@@ -483,7 +471,7 @@ Select(void *process)
 
           if (ntsteps2 == 0 && nfiles > 1)
             {
-              lconstvars = false;
+              hasConstVars = false;
               for (varID = 0; varID < nvars2; ++varID) vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
             }
 
@@ -498,17 +486,17 @@ Select(void *process)
                   SELINFO_GET_VAL(timestep, i, &ptimestep);
                   if (ptimestep < 0)
                     {
-                      if (nfiles != 1) cdoAbort("Negative timesteps only supported with one input stream!");
-                      if (ntsteps < 0 && cdoAssertFilesOnly())
+                      if (nfiles != 1) cdo_abort("Negative timesteps only supported with one input stream!");
+                      if (ntsteps < 0 && cdo_assert_files_only())
                         {
                           int tsID = 0;
-                          while (cdoStreamInqTimestep(streamID1, tsID)) tsID++;
+                          while (cdo_stream_inq_timestep(streamID1, tsID)) tsID++;
                           ntsteps = tsID;
-                          if (Options::cdoVerbose) cdoPrint("Found %d timesteps", ntsteps);
+                          if (Options::cdoVerbose) cdo_print("Found %d timesteps", ntsteps);
                         }
                       if (ntsteps > 0)
                         {
-                          if (Options::cdoVerbose) cdoPrint("timestep %d changed to %d", ptimestep, ptimestep + ntsteps + 1);
+                          if (Options::cdoVerbose) cdo_print("timestep %d changed to %d", ptimestep, ptimestep + ntsteps + 1);
                           ptimestep += ntsteps + 1;
                           SELINFO_DEF_VAL(timestep, i, &ptimestep);
                         }
@@ -523,54 +511,50 @@ Select(void *process)
                 }
             }
 
-          if (!lcopy)
-            {
-              auto gridsizemax = vlistGridsizeMax(vlistID1);
-              if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-              array.resize(gridsizemax);
-            }
-
           SELINFO_GET_VAL(startdate, 0, &startdate);
           SELINFO_GET_VAL(enddate, 0, &enddate);
-          if (SELINFO_NVAL(startdate)) fstartdate = datestrToDouble(startdate, 0);
-          if (SELINFO_NVAL(enddate)) fenddate = datestrToDouble(enddate, 1);
+          if (SELINFO_NVAL(startdate)) fstartdate = date_str_to_double(startdate, 0);
+          if (SELINFO_NVAL(enddate)) fenddate = date_str_to_double(enddate, 1);
         }
       else
         {
-          vlistCompare(vlistID0, vlistID1, CMP_ALL);
+          vlist_compare(vlistID0, vlistID1, CMP_ALL);
         }
 
       if (nvars2 == 0)
         {
-          cdoWarning("No variable selected!");
+          cdo_warning("No variable selected!");
           goto END_LABEL;
         }
 
-      if (lcopy_const) vardata2.resize(nvars2);
+      if (copyConstVars) vardata2.resize(nvars2);
 
-      bool lstop = false;
+      auto stopReading = false;
       int tsID1 = 0;
-      while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
+      while (true)
         {
-          timestep++;
-          bool copytimestep = true;
+          const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID1);
+          if (nrecs == 0) break;
 
-          if (ltimsel)
+          timestep++;
+          auto copyTimestep = true;
+
+          if (doTimeSel)
             {
-              copytimestep = false;
+              copyTimestep = false;
 
               if (operatorID == SELECT && SELINFO_NVAL(timestep) > 0)
                 {
                   if (timestep > tsmax)
                     {
-                      lstop = true;
+                      stopReading = true;
                       break;
                     }
                 }
 
-              dtlist.taxisInqTimestep(taxisID1, 0);
-              const auto vdate = dtlist.getVdate(0);
-              const auto vtime = dtlist.getVtime(0);
+              dtlist.taxis_inq_timestep(taxisID1, 0);
+              const auto vdate = dtlist.get_vdate(0);
+              const auto vtime = dtlist.get_vtime(0);
               int second;
               cdiDecodeDate(vdate, &year, &month, &day);
               cdiDecodeTime(vtime, &hour, &minute, &second);
@@ -584,10 +568,10 @@ Select(void *process)
 
               timestep_of_year++;
 
-              if (SELINFO_CHECK(timestep)) copytimestep = true;
-              if (SELINFO_CHECK(timestep_of_year)) copytimestep = true;
+              if (SELINFO_CHECK(timestep)) copyTimestep = true;
+              if (SELINFO_CHECK(timestep_of_year)) copyTimestep = true;
 
-              if (!copytimestep && SELINFO_NVAL(date) == 0 && SELINFO_NVAL(timestep) == 0 && SELINFO_NVAL(timestep_of_year) == 0
+              if (!copyTimestep && SELINFO_NVAL(date) == 0 && SELINFO_NVAL(timestep) == 0 && SELINFO_NVAL(timestep_of_year) == 0
                   && SELINFO_NVAL(dom) == 0)
                 {
                   bool lseason = false, lyear = false, lmonth = false, lday = false, lhour = false, lminute = false;
@@ -599,20 +583,20 @@ Select(void *process)
                   if (SELINFO_NVAL(hour) == 0 || SELINFO_CHECK(hour)) lhour = true;
                   if (SELINFO_NVAL(minute) == 0 || SELINFO_CHECK(minute)) lminute = true;
 
-                  if (lseason && lyear && lmonth && lday && lhour && lminute) copytimestep = true;
+                  if (lseason && lyear && lmonth && lday && lhour && lminute) copyTimestep = true;
                 }
 
-              const double fdate = ((double) vdate) + ((double) vtime) / 1000000.;
+              const double fdate = ((double) vdate) + ((double) vtime) / 1000000.0;
 
               if (SELINFO_NVAL(enddate))
                 {
-                  copytimestep = (fdate <= fenddate);
+                  copyTimestep = (fdate <= fenddate);
                   if (fdate > fenddate)
                     {
                       SELINFO_DEF_FLAG(enddate, 0, true);
                       if (operatorID == SELECT)
                         {
-                          lstop = true;
+                          stopReading = true;
                           break;
                         }
                     }
@@ -620,78 +604,78 @@ Select(void *process)
 
               if (SELINFO_NVAL(startdate))
                 {
-                  copytimestep = (fdate >= fstartdate);
+                  copyTimestep = (fdate >= fstartdate);
                   if (fdate >= fstartdate) SELINFO_DEF_FLAG(startdate, 0, true);
                 }
 
               if (SELINFO_NVAL(date))
                 {
-                  const auto datetimeString = datetimeToString(vdate, vtime);
+                  const auto datetimeString = datetime_to_string(vdate, vtime);
                   date = datetimeString.c_str();
-                  if (SELINFO_CHECK_DATE(date)) copytimestep = true;
+                  if (SELINFO_CHECK_DATE(date)) copyTimestep = true;
                 }
 
               if (SELINFO_NVAL(dom))
                 {
-                  const auto domString = domToString(vdate);
+                  const auto domString = dom_to_string(vdate);
                   dom = domString.c_str();
-                  if (SELINFO_CHECK_DATE(dom)) copytimestep = true;
+                  if (SELINFO_CHECK_DATE(dom)) copyTimestep = true;
                 }
 
-              if (operatorID == DELETE) copytimestep = !copytimestep;
+              if (operatorID == DELETE) copyTimestep = !copyTimestep;
 
-              if (copytimestep && indf == 0 && tsID1 == 0) lcopy_const = false;
+              if (copyTimestep && indf == 0 && tsID1 == 0) copyConstVars = false;
             }
 
-          if (copytimestep)
+          if (copyTimestep)
             {
-              taxisCopyTimestep(taxisID2, taxisID1);
+              cdo_taxis_copy_timestep(taxisID2, taxisID1);
               if (streamID2 == CDO_STREAM_UNDEF)
                 {
-                  const bool lasttimestep = (nfiles == 1) && (ntsteps2 > 1) && (ntsteps2 == (tsID1 + 1));
-                  if (lasttimestep && tsID2 == 0) ntsteps2 = 1;
+                  const auto isLastTimestep = ((nfiles == 1) && (ntsteps2 > 1) && (ntsteps2 == (tsID1 + 1)));
+                  if (isLastTimestep && tsID2 == 0) ntsteps2 = 1;
                   if (ntsteps2 == 0 || ntsteps2 == 1) vlistDefNtsteps(vlistID2, ntsteps2);
-                  streamID2 = cdoOpenWrite(nfiles);
+                  streamID2 = cdo_open_write(nfiles);
                   vlistDefTaxis(vlistID2, taxisID2);
-                  cdoDefVlist(streamID2, vlistID2);
+                  cdo_def_vlist(streamID2, vlistID2);
                 }
-              cdoDefTimestep(streamID2, tsID2);
+              cdo_def_timestep(streamID2, tsID2);
 
               for (int recID = 0; recID < nrecs; ++recID)
                 {
-                  cdoInqRecord(streamID1, &varID, &levelID);
+                  cdo_inq_record(streamID1, &varID, &levelID);
                   if (vlistInqFlag(vlistID0, varID, levelID) == true)
                     {
-                      if (lconstvars && tsID2 > 0 && tsID1 == 0)
+                      if (hasConstVars && tsID2 > 0 && tsID1 == 0)
                         if (varList1[varID].timetype == TIME_CONSTANT) continue;
 
                       const auto varID2 = vlistFindVar(vlistID2, varID);
                       const auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
-                      if (lcopy_const && tsID2 == 0) write_const_vars(streamID2, varList2, varID2, vardata2);
+                      if (copyConstVars && tsID2 == 0) write_const_vars(streamID2, varList2, varID2, vardata2);
 
-                      cdoDefRecord(streamID2, varID2, levelID2);
-                      if (lcopy)
+                      cdo_def_record(streamID2, varID2, levelID2);
+                      if (dataIsUnchanged)
                         {
-                          cdoCopyRecord(streamID2, streamID1);
+                          cdo_copy_record(streamID2, streamID1);
                         }
                       else
                         {
-                          size_t nmiss;
-                          cdoReadRecord(streamID1, array.data(), &nmiss);
-                          cdoWriteRecord(streamID2, array.data(), nmiss);
+                          field.init(varList1[varID]);
+                          cdo_read_record(streamID1, field);
+                          cdo_write_record(streamID2, field);
                         }
                     }
                 }
 
-              if (lcopy_const && tsID2 == 0) write_const_vars(streamID2, varList2, nvars2, vardata2);
+              if (copyConstVars && tsID2 == 0) write_const_vars(streamID2, varList2, nvars2, vardata2);
 
               tsID2++;
             }
-          else if (lcopy_const && indf == 0 && tsID1 == 0)
+          else if (copyConstVars && indf == 0 && tsID1 == 0)
             {
               for (int recID = 0; recID < nrecs; ++recID)
                 {
-                  cdoInqRecord(streamID1, &varID, &levelID);
+                  cdo_inq_record(streamID1, &varID, &levelID);
                   if (vlistInqFlag(vlistID0, varID, levelID) == true)
                     {
                       const auto varID2 = vlistFindVar(vlistID2, varID);
@@ -701,7 +685,7 @@ Select(void *process)
                           const auto gridsize = varList2[varID2].gridsize;
                           if (levelID == 0) vardata2[varID2].resize(gridsize * varList2[varID2].nlevels);
                           size_t nmiss;
-                          cdoReadRecord(streamID1, &vardata2[varID2][gridsize * levelID2], &nmiss);
+                          cdo_read_record(streamID1, &vardata2[varID2][gridsize * levelID2], &nmiss);
                         }
                     }
                 }
@@ -710,9 +694,9 @@ Select(void *process)
           tsID1++;
         }
 
-      cdoStreamClose(streamID1);
+      cdo_stream_close(streamID1);
 
-      if (lstop) break;
+      if (stopReading) break;
     }
 
 END_LABEL:
@@ -732,14 +716,14 @@ END_LABEL:
   SELINFO_CHECK_FLAG(date);
   SELINFO_CHECK_FLAG(dom);
 
-  if (streamID2 != CDO_STREAM_UNDEF) cdoStreamClose(streamID2);
+  if (streamID2 != CDO_STREAM_UNDEF) cdo_stream_close(streamID2);
 
   vlistDestroy(vlistID0);
   vlistDestroy(vlistID2);
 
-  if (tsID2 == 0) cdoAbort("No timesteps selected!");
+  if (tsID2 == 0) cdo_abort("No timesteps selected!");
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
