@@ -87,7 +87,7 @@ static void
 boundboxFromCorners(size_t size, size_t nc, const Varray<double> &corner_lon, const Varray<double> &corner_lat, float *bound_box)
 {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(bound_box, corner_lat, corner_lon, nc, size)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < size; ++i)
     {
@@ -116,7 +116,7 @@ boundboxFromCenter(bool lonIsCyclic, size_t size, size_t nx, size_t ny, const Va
                    const Varray<double> &center_lat, float *bound_box)
 {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(lonIsCyclic, size, nx, ny, center_lon, center_lat, bound_box)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t n = 0; n < size; n++)
     {
@@ -191,7 +191,7 @@ check_lon_range(const char *txt, size_t nlons, Varray<double> &lons)
       double minval = 1.e36;
       double maxval = -1.e36;
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlons, lons) reduction(min : minval) reduction(max : maxval)
+#pragma omp parallel for default(shared) schedule(static) reduction(min : minval) reduction(max : maxval)
 #endif
       for (size_t i = 0; i < nlons; ++i)
         {
@@ -203,7 +203,7 @@ check_lon_range(const char *txt, size_t nlons, Varray<double> &lons)
     }
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlons, lons)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < nlons; ++i)
     {
@@ -226,7 +226,7 @@ check_lat_range(const char *txt, size_t nlats, Varray<double> &lats)
       double minval = 1.e36;
       double maxval = -1.e36;
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlats, lats) reduction(min : minval) reduction(max : maxval)
+#pragma omp parallel for default(shared) schedule(static) reduction(min : minval) reduction(max : maxval)
 #endif
       for (size_t i = 0; i < nlats; ++i)
         {
@@ -238,7 +238,7 @@ check_lat_range(const char *txt, size_t nlats, Varray<double> &lats)
     }
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlats, lats)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < nlats; ++i)
     {
@@ -253,12 +253,12 @@ check_lon_boundbox_range(size_t nlons, float *bound_box)
   assert(bound_box != nullptr);
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlons, bound_box)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t n = 0; n < nlons; ++n)
     {
       const auto n4 = n << 2;
-      if (fabsf(bound_box[n4 + 3] - bound_box[n4 + 2]) > PI_f)
+      if (std::fabs(bound_box[n4 + 3] - bound_box[n4 + 2]) > PI_f)
         {
           bound_box[n4 + 2] = 0.0f;
           bound_box[n4 + 3] = PI2_f;
@@ -272,7 +272,7 @@ check_lat_boundbox_range(size_t nlats, float *bound_box, Varray<double> &lats)
   assert(bound_box != nullptr);
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlats, bound_box, lats)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t n = 0; n < nlats; ++n)
     {
@@ -445,7 +445,7 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
   // Initialize logical mask
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(gridsize, grid)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < gridsize; ++i) grid.mask[i] = 1;
 
@@ -885,7 +885,7 @@ remap_gradients(RemapGrid &grid, const Varray<short> &mask, const Varray<T> &arr
   auto ny = grid.dims[1];
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(grid_size, gradients, grid, nx, ny, array, mask)
+#pragma omp parallel for default(shared)
 #endif
   for (size_t n = 0; n < grid_size; ++n)
     {
@@ -1051,7 +1051,7 @@ remap_gradients(RemapGrid &grid, const Field &field, RemapGradients &gradients)
   auto grid_size = grid.size;
   Varray<short> mask(grid_size);
 #ifdef _OPENMP
-#pragma omp parallel for default(none) schedule(static) shared(grid_size, grid, mask)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < grid_size; ++i) mask[i] = (grid.mask[i] > 0);
 
